@@ -26,8 +26,8 @@
 
 import pwem
 
-__version__ = '3.0.12'
-_logo = "icon.gif"
+__version__ = '3.1.0'
+_logo = "icon.png"
 _references = []
 
 
