@@ -36,7 +36,6 @@ from pwem.viewers.showj import runJavaIJapp
 from pyworkflow.gui import *
 from pyworkflow.gui.tree import TreeProvider
 from pyworkflow.gui.dialog import ListDialog, ToolbarListDialog, showInfo
-import pyworkflow.config as conf
 import pyworkflow.viewer as pwviewer
 import pyworkflow.utils as pwutils
 from pyworkflow.plugin import Domain
@@ -52,10 +51,12 @@ class TiltSeriesTreeProvider(TreeProvider):
     COL_TS = 'Tilt series'
     COL_TI = 'Path'
     COL_TI_ANGLE = 'Tilt angle'
+    COL_TI_ENABLED = 'Included'
     COL_TI_ACQ_ORDER = 'Order'
     COL_TI_DEFOCUS_U = 'Defocus U (A)'
     COL_TI_DOSE = "Accum. dose"
     ORDER_DICT = {COL_TI_ANGLE: '_tiltAngle',
+                  COL_TI_ENABLED: '_objEnabled',
                   COL_TI_DEFOCUS_U: '_ctfModel._defocusU',
                   COL_TI_DOSE: '_acquisition._accumDose'}
 
@@ -81,7 +82,10 @@ class TiltSeriesTreeProvider(TreeProvider):
             tsObj._parentObject = None
             objects.append(tsObj)
             for ti in ts.iterItems(orderBy=orderBy, direction=direction):
+
                 tiObj = ti.clone()
+                # For some reason .clone() does not clone the enabled nor the creation time
+                tiObj.setEnabled(ti.isEnabled())
                 tiObj._allowsSelection = False
                 tiObj._parentObject = tsObj
                 objects.append(tiObj)
@@ -99,6 +103,7 @@ class TiltSeriesTreeProvider(TreeProvider):
             (self.COL_TS, 100),
             (self.COL_TI_ACQ_ORDER, 100),
             (self.COL_TI_ANGLE, 100),
+            (self.COL_TI_ENABLED, 100),
             (self.COL_TI_DOSE, 100),
             (self.COL_TI, 400),
         ]
@@ -129,11 +134,12 @@ class TiltSeriesTreeProvider(TreeProvider):
             key = '%s.%s' % (tsId, objId)
             text = objId
 
-            dose = obj.getAcquisition().getAccumDose() if hasattr(obj, '_accumDose') else None
+            dose = obj.getAcquisition().getAccumDose() if hasattr(obj.getAcquisition(), '_accumDose') else None
             adqOrder = obj.getAcquisitionOrder() if hasattr(obj, '_acqOrder') else None
 
             values = [str("%d" % adqOrder) if adqOrder is not None else "",
                       str("%0.2f" % obj.getTiltAngle()),
+                      str(obj.isEnabled()),
                       round(dose, 2) if dose is not None else "",
                       "%d@%s" % (obj.getLocation()[0] or 1, obj.getLocation()[1])]
 
@@ -191,8 +197,8 @@ class TiltSeriesDialogView(pwviewer.View):
         self._provider = TiltSeriesTreeProvider(protocol, tiltSeries)
 
     def show(self):
-        dlg = ListDialog(self._tkParent, 'TiltSeries display', self._provider,
-                         allowSelect=False, cancelButton=True)
+        ListDialog(self._tkParent, 'Tilt series viewer', self._provider,
+                   allowSelect=False, cancelButton=True)
 
 
 class TomogramsTreeProvider(TreeProvider):
@@ -627,7 +633,6 @@ class CtfEstimationTreeProvider(TreeProvider, ttk.Treeview):
     """
     COL_CTF_SERIE = 'Tilt Series'
     COL_TILT_ANG = 'Tilt Angle'
-    CRITERIA_1 = 'Status'
     COL_CTF_EST_DEFOCUS_U = 'DefocusU (A)'
     COL_CTF_EST_DEFOCUS_V = 'DefocusV (A)'
     COL_CTF_EST_AST = 'Astigmatism (A)'
@@ -686,7 +691,6 @@ class CtfEstimationTreeProvider(TreeProvider, ttk.Treeview):
         cols = [
             (self.COL_CTF_SERIE, 100),
             (self.COL_TILT_ANG, 100),
-            (self.CRITERIA_1, 60),
             (self.COL_CTF_EST_DEFOCUS_U, 100),
             (self.COL_CTF_EST_DEFOCUS_V, 100),
             (self.COL_CTF_EST_AST, 150),
@@ -711,8 +715,9 @@ class CtfEstimationTreeProvider(TreeProvider, ttk.Treeview):
             key = obj.getTsId()
             text = obj.getTsId()
             # TODO: show avg defocus for TomoSeries
-            values = ['', CTFSerieStates.OK if obj.getIsDefocusUDeviationInRange()
-                      else CTFSerieStates.FAILED]
+            values = ['',
+                      #CTFSerieStates.OK if obj.isEnabled() else CTFSerieStates.FAILED
+                      ]
             opened = False
             selected = obj.isEnabled()
         else:  # CTFTomo
@@ -724,8 +729,7 @@ class CtfEstimationTreeProvider(TreeProvider, ttk.Treeview):
             phSh = obj.getPhaseShift() if obj.hasPhaseShift() else 0
 
             values = [str("%0.2f" % tiltAngle),
-                      CTFSerieStates.OK if obj.getIsDefocusUDeviationInRange()
-                      else CTFSerieStates.FAILED,
+                      #CTFSerieStates.OK if obj.isEnabled() else CTFSerieStates.FAILED,
                       str("%d" % obj.getDefocusU()),
                       str("%d" % obj.getDefocusV()),
                       str("%d" % ast),
@@ -747,7 +751,7 @@ class CtfEstimationTreeProvider(TreeProvider, ttk.Treeview):
         }
         if isinstance(obj, tomo.objects.CTFTomoSeries):
             tags = CTFSerieStates.UNCHECKED
-            if not (obj.getIsDefocusUDeviationInRange() and obj.getIsDefocusVDeviationInRange()):
+            if not obj.isEnabled():
                 obj.setEnabled(True)
                 tags = CTFSerieStates.CHECKED
                 self._checkedItems += 1
@@ -1011,6 +1015,8 @@ class CtfEstimationListDialog(ListDialog):
                     ctfId = int(itemSelected.split('.')[-1])
                     # TODO: sort ctfSerie by id
                     fig = self._show2DPLot(ctfSerie, ctfId)
+                    if fig is None:
+                        return
                     canvas = FigureCanvasTkAgg(fig, master=plotterPanel)
                     canvas.draw()
                     canvas.get_tk_widget().pack(fill=BOTH, expand=0)
@@ -1025,12 +1031,15 @@ class CtfEstimationListDialog(ListDialog):
         phShList = []
         resList = []
 
+        for ts in self._inputSetOfTiltSeries:
+            if ts.getTsId() == itemSelected:
+                for item in ts:
+                    angList.append(item.getTiltAngle())
+                break
+
         for ctfSerie in self.provider.getCTFSeries():
-            ts = ctfSerie.getTiltSeries()
             if ctfSerie.getTsId() == itemSelected:
                 for item in ctfSerie.iterItems(orderBy='id'):
-                    index = int(item.getIndex())
-                    angList.append(int(ts[index].getTiltAngle()))
                     defocusUList.append(item.getDefocusU())
                     defocusVList.append(item.getDefocusV())
                     phShList.append(
